import sys
from contextlib import contextmanager

from emport import import_file
import dessert

import pytest
from _pytest.assertion.rewrite import AssertionRewritingHook as PytestRewriteHook


@contextmanager
def _disable_pytest_rewriting():
    old_meta_path = sys.meta_path[:]
    try:
        for index, plugin in reversed(list(enumerate(sys.meta_path))):
            if isinstance(plugin, PytestRewriteHook):
                sys.meta_path.pop(index)
        yield
    finally:
        sys.meta_path[:] = old_meta_path



def test_dessert_interop(tmpdir):
    path = tmpdir.join('testme.py')
    with path.open('w') as f:
        f.write("""def f():
    a = 1
    b = 2
    assert a == b
""")

    with _disable_pytest_rewriting():
        with dessert.rewrite_assertions_context():
            mod = import_file(str(path))
    with pytest.raises(AssertionError) as caught:
        mod.f()

    assert '1 == 2' in str(caught.value)



import itertools
import os
import sys
import uuid

import logbook

import pytest

_filename_generator = ("module{0}.py".format(i) for i in itertools.count())


@pytest.fixture(scope='session', autouse=True)
def configure_logging():
    logbook.StderrHandler().push_application()


@pytest.fixture(autouse=True)
def preserve_sys_modules(request):
    old_modules = sys.modules.copy()
    old_path = sys.path[:]

    @request.addfinalizer
    def cleanup():
        sys.modules.clear()
        sys.modules.update(old_modules)
        sys.path[:] = old_path


def create_module_file(directory, filename=None):
    directory = str(directory)
    if not os.path.isdir(directory):
        os.makedirs(directory)
    id = repr(uuid.uuid1())
    if filename is None:
        filename = next(_filename_generator)
    full_filename = os.path.join(directory, filename)
    with open(full_filename, "w") as f:
        f.write("value = {0!r}".format(id))
    return full_filename, id


@pytest.fixture
def module_file_factory(request):
    return create_module_file
